# -*- coding: utf-8 -*-
from django import forms
from django.contrib.contenttypes.models import ContentType
from django.template.loader import render_to_string


class ObjectFilterValuesWidget(forms.SelectMultiple):

    def render(self, name, value, *args, **kwargs):
        option_list = []
        values = value
        try:
            values = eval(values)
        except:
            values = []

        if 'content_type' in self.attrs:
            content_type_id = self.attrs['content_type']
        else:
            content_type_id = None

        if isinstance(values, list):
            if content_type_id:
                cnt_type = ContentType.objects.get(pk=content_type_id)
                model_class = cnt_type.model_class()
                option_list = [(o.id, o.__str__()) for o in model_class._default_manager.distinct()]
        else:
            option_list.insert(0, ('0', ''))
            values = '0'

        return render_to_string("object_filter_values_widget.html", {'option_list': option_list, 'selected': values})



# -*- coding: utf-8 -*-
"""
Settings for Django Authorization are all namespaced in the DJ_AUTH setting.
For example your project's `settings.py` file might look like this:

DJ_AUTH = {
    'content_type_exclude': ('contenttypes.contenttype', 'sessions.session', 'sites.site',
                             'auth.user', 'auth.group', 'auth.permission', 'admin.logentry',
                             'dj_auth.objectfilter',
    ),
    'content_type_include': (),
    'global_fields_exclude':  ('user', ),
    'related_filter_fields_exclude': {'auth.user': ('groups', ), },
}

"""
from django.conf import settings


DEFAULT = {
    'content_type_exclude': (),
    'content_type_include': (),
    'global_fields_exclude': (),
    'related_filter_fields_exclude': {},
}

DJ_AUTH = getattr(settings, 'DJ_AUTH', DEFAULT)



# -*- coding: utf-8 -*-
from django.conf.urls import url

from .views import ObjectFilterValuesWidgetView

urlpatterns = [url(r'^loadvalues/(?P<content_type_id>[0-9]+)/$', ObjectFilterValuesWidgetView, name='loadvalues'), ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-08-11 12:39
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dj_auth', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='objectfilter',
            name='content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='dj_auth_objectfilter_content_type', to='contenttypes.ContentType', verbose_name='Class'),
        ),
        migrations.AlterField(
            model_name='objectfilter',
            name='filter_type',
            field=models.SmallIntegerField(choices=[(0, 'Equals'), (1, 'Not Equals')], verbose_name='Filter Type'),
        ),
        migrations.AlterField(
            model_name='objectfilter',
            name='users',
            field=models.ManyToManyField(related_name='dj_auth_objectfilter_users', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-29 12:48
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ObjectFilter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filter_type', models.SmallIntegerField(choices=[(0, 'Filter'), (1, 'Exclude')], verbose_name='Filter Type')),
                ('filter_values', models.CharField(blank=True, max_length=250, null=True, verbose_name='Values')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='djauth_objectfilter_content_type', to='contenttypes.ContentType', verbose_name='Class')),
                ('users', models.ManyToManyField(related_name='djauth_objectfilter_users', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Data filter',
                'verbose_name_plural': 'Data filter',
            },
        ),
    ]



# -*- coding: utf-8 -*-
import logging
logger = logging.getLogger(__name__)

from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse

from .models import ObjectFilter
from .widgets import ObjectFilterValuesWidget


class ObjectFilterListMixin(object):

    def get_queryset(self):
        queryset = super(ObjectFilterListMixin, self).get_queryset()

        if isinstance(self.request.user, AnonymousUser):
            user = None
        else:
            user = self.request.user

        filter_query, exclude_query = ObjectFilter.build_filter(user=user, content_type=ContentType.objects.get_for_model(self.model), session=self.request.session)

        if exclude_query:
            queryset = queryset.exclude(exclude_query).distinct()
        if filter_query:
            queryset = queryset.filter(filter_query).distinct()
        return queryset


class _ObjectFilterDetailMixin(object):
    def get_object(self):
        obj = super(_ObjectFilterDetailMixin, self).get_object()

        if isinstance(self.request.user, AnonymousUser):
            user = None
        else:
            user = self.request.user

        filter_query, exclude_query = ObjectFilter.build_filter(user=user, content_type=ContentType.objects.get_for_model(self.model), session=self.request.session)

        if exclude_query:
            if self.model.objects.exclude(exclude_query).filter(pk=obj.id).exists():
                return obj
            else:
                raise PermissionDenied
        if filter_query:
            if self.model.objects.filter(filter_query).filter(pk=obj.id).exists():
                return obj
            else:
                raise PermissionDenied

        return obj


class ObjectFilterDetailMixin(_ObjectFilterDetailMixin):
    pass


class ObjectFilterUpdateMixin(_ObjectFilterDetailMixin):
    pass


class ObjectFilterDeleteMixin(_ObjectFilterDetailMixin):
    pass


def ObjectFilterValuesWidgetView(request, content_type_id=None, field_name=None, values=None):
    widget = ObjectFilterValuesWidget(attrs={'content_type': content_type_id, 'field_name': field_name, 'values': values})
    return HttpResponse(widget.render(field_name, [], ))



# -*- coding: utf-8 -*-
import logging
logger = logging.getLogger(__name__)

from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.db import models
from django.db.models import Q
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import AnonymousUser

from .settings import DEFAULT
from .constants import FILTER_TYPE, FILTER_ID, EXCLUDE_ID


class ObjectFilter(models.Model):
    users = models.ManyToManyField(settings.AUTH_USER_MODEL, blank=False, related_name='%(app_label)s_%(class)s_users', verbose_name=_('User'))
    content_type = models.ForeignKey(ContentType, null=False, blank=False, related_name='%(app_label)s_%(class)s_content_type', verbose_name=_('Class'))
    filter_type = models.SmallIntegerField(choices=FILTER_TYPE, null=False, blank=False, verbose_name=_('Filter Type'))
    filter_values = models.CharField(max_length=250, null=True, blank=True, verbose_name=_('Values'))

    class Meta:
        verbose_name = _('Data filter')
        verbose_name_plural = _('Data filter')

    def __str__(self):
        return "%s - %s" % (', '.join([str(i) for i in self.users.all()]), self.content_type)

    @classmethod
    def _check_filter_values(cls, record=None):
        cls.filter_values = eval(record.filter_values)

        if not isinstance(cls.filter_values, list):
            raise ValueError(_('Filter Values is not a List: %s' % record.filter_values))

    @classmethod
    def _update_filter_dict(cls, model=None, field_name='id', filter_type=FILTER_ID, values=[]):
        if model and model._meta.label_lower not in cls._filter_dict:
            cls._filter_dict[model._meta.label_lower] = {FILTER_ID: {}, EXCLUDE_ID: {}}
        if values:
            if '%s__in' % field_name in cls._filter_dict[model._meta.label_lower][filter_type]:
                cls._filter_dict[model._meta.label_lower][filter_type]['%s__in' % field_name].extend(values)
                cls._filter_dict[model._meta.label_lower][filter_type]['%s__in' % field_name] = list(set(cls._filter_dict[model._meta.label_lower][filter_type]['%s__in' % field_name]))
            else:
                cls._filter_dict[model._meta.label_lower][filter_type]['%s__in' % field_name] = values
        else:
            cls._filter_dict[model._meta.label_lower][filter_type]['%s__isnull' % field_name] = True

    @classmethod
    def _create_filter_dict(cls, user=None, session=None):
        DJ_AUTH = getattr(settings, 'DJ_AUTH', DEFAULT)

        cls._filter_dict = {}

        for record in ObjectFilter.objects.filter(users=user):
            cls._check_filter_values(record)

            content_type = ContentType.objects.get(pk=record.content_type_id)
            model_class = content_type.model_class()

            # Create filter for the own class
            cls._update_filter_dict(model_class, field_name='id', filter_type=record.filter_type, values=eval(record.filter_values))

            related_models = []
            for field in model_class._meta.get_fields():
                if field.is_relation and field.related_model:
                    if field.related_model not in related_models:
                        related_models.append(field.related_model)

            # Create filter for the related classes
            for related_model in related_models:
                for field in related_model._meta.get_fields():
                    if field.is_relation and field.related_model and not field.auto_created and field.related_model == model_class:
                        if DJ_AUTH['global_fields_exclude'] and field.name in DJ_AUTH['global_fields_exclude']:
                            continue

                        if DJ_AUTH['related_filter_fields_exclude'] and related_model._meta.label_lower in DJ_AUTH['related_filter_fields_exclude'] and field.name in DJ_AUTH['related_filter_fields_exclude'][related_model._meta.label_lower]:
                            continue

                        cls._update_filter_dict(related_model, field_name=field.name, filter_type=record.filter_type, values=eval(record.filter_values))

        if 'django.contrib.sessions' in settings.INSTALLED_APPS and session:
            session['dj_auth_filter_dict'] = str(cls._filter_dict)

    @classmethod
    def build_filter(cls, user=None, content_type=None, session=None):
        DJ_AUTH = getattr(settings, 'DJ_AUTH', DEFAULT)

        if 'django.contrib.sessions' in settings.INSTALLED_APPS and session and 'dj_auth_filter_dict' in session:
            cls._filter_dict = eval(session['dj_auth_filter_dict'])
        else:
            cls._create_filter_dict(user=user, session=session)

        filter_query = ''
        exclude_query = ''

        model_class = content_type.model_class()
        # filter for the own class
        if model_class._meta.label_lower in cls._filter_dict:
            for k, v in cls._filter_dict[model_class._meta.label_lower][FILTER_ID].items():
                filter_query += "Q(**{'%s': %s}) | " % (k, v)

            if filter_query.endswith('| '):
                filter_query = filter_query[:-2]

            for k, v in cls._filter_dict[model_class._meta.label_lower][EXCLUDE_ID].items():
                exclude_query += "Q(**{'%s': %s}) | " % (k, v)

            if exclude_query.endswith('| '):
                exclude_query = exclude_query[:-2]
        else:
            # if the class is not set as filter, check FK an M2M of the class
            related_models = {}
            for field in model_class._meta.get_fields():
                if field.is_relation and field.related_model and not field.auto_created:
                    if field.related_model._meta.label_lower in cls._filter_dict:
                        if DJ_AUTH['global_fields_exclude'] and field.name in DJ_AUTH['global_fields_exclude']:
                            continue

                        if DJ_AUTH['related_filter_fields_exclude'] and model_class._meta.label_lower in DJ_AUTH['related_filter_fields_exclude'] and field.name in DJ_AUTH['related_filter_fields_exclude'][model_class._meta.label_lower]:
                            continue

                        if field.related_model not in related_models:
                            related_models[field.related_model] = []
                        related_models[field.related_model].append(field.name)

            for related_model, field_name_list in related_models.items():
                for filter_text, filter_value in cls._filter_dict[related_model._meta.label_lower][FILTER_ID].items():
                    for field_name in field_name_list:
                        filter_query += "Q(**{'%s__%s': %s}) | " % (field_name, filter_text, filter_value)

                for filter_text, filter_value in cls._filter_dict[related_model._meta.label_lower][EXCLUDE_ID].items():
                    for field_name in field_name_list:
                        exclude_query += "Q(**{'%s__%s': %s}) | " % (field_name, filter_text, filter_value)

            if filter_query.endswith('| '):
                filter_query = filter_query[:-2]

            if exclude_query.endswith('| '):
                exclude_query = exclude_query[:-2]

        if filter_query:
            filter_query = eval(filter_query)

        if exclude_query:
            exclude_query = eval(exclude_query)

        logger.debug("%s filter %s" % (model_class._meta.label_lower, filter_query))
        logger.debug("%s exclude %s" % (model_class._meta.label_lower, exclude_query))

        return filter_query, exclude_query


class ObjectFilterQuerySetMixin(object):

    def apply_user_object_filter(self, user=None, session=None):
        queryset = self
        if user and not isinstance(user, AnonymousUser):
            filter_query, exclude_query = ObjectFilter.build_filter(user, ContentType.objects.get_for_model(self.model), session=session)
            if exclude_query:
                queryset = self.exclude(exclude_query).distinct()
            if filter_query:
                queryset = self.filter(filter_query).distinct()

        return queryset

    def apply_user_generic_objects_filter(self, user=None, session=None):
        queryset = self
        classes_to_check = []
        for record in ObjectFilter.objects.filter(users=user):
            cnt_type = ContentType.objects.get(pk=record.content_type_id)
            filter_class = cnt_type.model_class()

            # classes_to_check.extend([o.related_model for o in filter_class._meta.get_all_related_objects()])
            for rel in filter_class._meta.get_fields():
                if (rel.one_to_many or rel.one_to_one) and rel.auto_created and not rel.concrete:
                    classes_to_check.append(rel.related_model)
            # classes_to_check.extend([o.related_model for o in filter_class._meta.get_all_related_many_to_many_objects()])
            for rel in filter_class._meta.get_fields(include_hidden=True):
                if rel.many_to_many and rel.auto_created:
                    classes_to_check.append(rel.related_model)
            classes_to_check.append(filter_class)

        f_query = ''
        e_query = ''
        classes_to_check = list(set(classes_to_check))
        for model_class in classes_to_check:
            cnt_type = ContentType.objects.get(app_label=model_class._meta.app_label, model=model_class._meta.model_name)
            filter_query, exclude_query = ObjectFilter.build_filter(user, content_type=cnt_type, session=session)
            if exclude_query:
                ids = model_class.objects.filter(exclude_query).values_list('id', flat=True)
                e_query += "Q(**{'content_type_id': %s, 'object_id__in': %s}) | " % (cnt_type.id, list(set(ids)))
            if filter_query:
                ids = model_class.objects.filter(filter_query).values_list('id', flat=True)
                f_query += "Q(**{'content_type_id': %s, 'object_id__in': %s}) | " % (cnt_type.id, list(set(ids)))

        if e_query:
            if e_query.endswith('| '):
                e_query = e_query[:-2]
            e_query = eval(e_query)
            queryset = queryset.exclude(e_query)

        if f_query:
            if f_query.endswith('| '):
                f_query = f_query[:-2]
            f_query = eval(f_query)
            queryset = queryset.filter(f_query)
        return queryset



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _

FILTER_ID = 0
FILTERT_TEXT = _('Equals')

EXCLUDE_ID = 1
EXCLUDE_TEXT = _('Not Equals')

FILTER_TYPE = (
    (FILTER_ID, FILTERT_TEXT),
    (EXCLUDE_ID, EXCLUDE_TEXT),
)



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.db.models import Q

from .settings import DEFAULT
from .models import ObjectFilter
from .widgets import ObjectFilterValuesWidget


class ObjectFilterFormMixin(object):

    class Meta:
        model = ObjectFilter
        fields = ('users', 'filter_type', 'content_type', 'filter_values')
        widgets = {'filter_values': ObjectFilterValuesWidget(),
                   }

    class Media:
        js = ('js/dj_auth.js', )

    def __init__(self, *args, **kwargs):
        DJ_AUTH = getattr(settings, 'DJ_AUTH', DEFAULT)

        super(ObjectFilterFormMixin, self).__init__(*args, **kwargs)

        content_type_queryset = self.fields['content_type'].queryset
        for record in DJ_AUTH['content_type_exclude']:
            app_label, model = record.split('.')
            content_type_queryset = content_type_queryset.exclude(app_label=app_label, model=model)

        filter_query = ''
        for record in DJ_AUTH['content_type_include']:
            app_label, model = record.split('.')
            filter_query += "Q(**{'app_label': '%s', 'model': '%s'}) |" % (app_label, model)

        if filter_query.endswith('|'):
            filter_query = filter_query[:-1]

        if filter_query:
            content_type_queryset = content_type_queryset.filter(eval(filter_query))

        self.fields['content_type'].queryset = content_type_queryset
        self.fields['content_type'].widget.attrs = {"id": 'dj_auth_content_type',
                                                    "onchange": 'refreshObjectFilterValuesWidget()', }

        if self.instance.pk:
            # Update
            self.fields['filter_values'].widget.attrs['content_type'] = self.instance.content_type.id

        self.fields['filter_values'].help_text = _('Hold down "Control", or "Command" on a Mac, to select more than one.')

    def clean(self):
        cleaned_data = super(ObjectFilterFormMixin, self).clean()
        return cleaned_data

    def clean_filter_values(self):
        data = self.cleaned_data['filter_values']
        if data:
            data = list(map(int, eval(data)))
        else:
            data = []
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = '1.2.1'

if sys.argv[-1] == 'publish':
    try:
        import wheel
        print("Wheel version: ", wheel.__version__)
    except ImportError:
        print('Wheel library missing. Please run "pip install wheel"')
        sys.exit()
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on github:")
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='dj_auth',
    version=version,
    description="""The goal of dj_auth is to limit data access for certain user over your django project.""",
    long_description=readme + '\n\n' + history,
    author='Robert Dollinger',
    author_email='robert.d@systent.it',
    url='https://gitlab.com/systent/dj_auth',
    packages=[
        'dj_auth',
    ],
    include_package_data=True,
    install_requires=[],
    license="BSD",
    zip_safe=False,
    keywords='dj_auth',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
